#include "config.h"

Config::Config( const string cfg_name ) {
  cf.open(cfg_name);
  swp_prm_name = get_value("swp_prm_name", "snr");
  string swp_prm_string = get_value("swp_prm", "");
  if (swp_prm_string==""){
    swp_prm_no = get_value("swp_prm_no", 1);
    double swp_prm_1    = get_value("swp_prm_1", 20.0);
    double swp_prm_N    = get_value("swp_prm_N", 20.0);
    swp_prm = linspace(swp_prm_1,swp_prm_N,swp_prm_no);
  }
  else {
    swp_prm = swp_prm_string.c_str();
    swp_prm_no = swp_prm.length();
  }
  swp_prm_cnt = 0;
}

int Config::get_value(string name, int def){
  int val;
  string type;
  int bytes;
  if (name==swp_prm_name)
    val = (int) itpp::round(swp_prm(swp_prm_cnt));
  else if (cf.exists(name)) {
    cf.info(name,type,bytes);
    if (type=="int32")
      cf >> Name(name) >> val;
    else 
      cout << type << "Config: wrong type: "+name << endl;
  }
  else {
    val = def;
    cout << "Config: no entry for \""+name+"\", using default value " << def << endl;
  }
  return val;
}

double Config::get_value(string name, double def){
  double val;
  if (name==swp_prm_name)
    val = swp_prm(swp_prm_cnt);
  else if (cf.exists(name))
    cf >> Name(name) >> val;
  else {
    val = def;
    cout << "Config: no entry for \""+name+"\", using default value " << def << endl;
  }
  return val;
}

string Config::get_value(const string name, const string def){
  string val;
  if (cf.exists(name))
      cf >> Name(name) >> val;
  else {
    val = def;
    cout << "Config: no entry for \""+name+"\", using default value " << def << endl;
  }
  return val;
}

/***************** Manual Config ********************/

Config_Man::Config_Man()
{
  //string null("");
  prm_int = vec_1(0);
  prm_dbl = vec_1(0.0);
  prm_str.set_size(1); //prm_str(0) = &null;
  prm_int_name.set_size(1); //prm_int_name(0) = &null;
  prm_dbl_name.set_size(1);
  prm_str_name.set_size(1);
}

Config_Man::~Config_Man()
{
  for (int i=0;i<prm_int_name.length()-1;i++)
    delete prm_int_name(i);
  for (int i=0;i<prm_dbl_name.length()-1;i++)
    delete prm_dbl_name(i);
  for (int i=0;i<prm_str_name.length()-1;i++)
    delete prm_str_name(i);
  for (int i=0;i<prm_str.length()-1;i++)
    delete prm_str(i);
}

int Config_Man::get_value(string name, int def)
{
  int val;
  prm_index = exists(name, def);
  if (prm_index+1)
    val = prm_int(prm_index);
  else
    val = def;
  return val;
}

double Config_Man::get_value(string name, double def)
{
  double val;
  prm_index = exists(name, def);
  if (prm_index+1)
    val = prm_dbl(prm_index);
  else
    val = def;
  return val;
}

string Config_Man::get_value(string name, string def)
{
  string val;
  prm_index = exists(name, def);
  if (prm_index+1)
    val = *prm_str(prm_index);
  else
    val = def;
  return val;
}

void Config_Man::set_value(string name, int val)
{
  string * str = new string(name);
  prm_int_name.ins(0, str);
  prm_int.ins     (0, val);
}

void Config_Man::set_value(string name, double val)
{
  string * str = new string(name);
  prm_dbl_name.ins(0, str);
  prm_dbl.ins     (0, val);
}

void Config_Man::set_value(string name, string val)
{
  string * str = new string(name);
  prm_str_name.ins(0, str);
  str = new string(val);
  prm_str.ins     (0, str);
}

int Config_Man::exists(string name, int val)
{ 
  for (int i=0;i<prm_int.length()-1;i++)
    if (name==*prm_int_name(i)) return i;
  return -1;
}

int Config_Man::exists(string name, double val)
{
  for (int i=0;i<prm_dbl.length()-1;i++)
    if (name==*prm_dbl_name(i)) return i;
  return -1;
}

int Config_Man::exists(string name, string val)
{
  for (int i=0;i<prm_str.length()-1;i++)
    if (name==*prm_str_name(i)) return i;
  return -1;
}
