#include "itcomm.h"
#include "config.h"
#include "time.h"

using namespace itpp;

int main(int argc, char *argv[])
{ 
  string id             = "default";                // id
  for (int i = 1; i < argc; i++) {
    if (!strcmp(argv[i],"-id")) id = (string) argv[i+1];
  }
  
  //Configuration file scan
  Config cfg(id+".cfg");
  string cfg_id = cfg.get_value("id", "default"); 
  cout << id+": Using configuration file \""+cfg_id+".cfg\"" << endl;

  BERC berc;
  Real_Timer tt;    //The timer used to measure the execution time
  tt.tic();

  RNG_randomize();

  int swp_prm_no = cfg.get_swp_prm_no();
  vec swp_prm = cfg.get_swp_prm();
  vec errorrate(swp_prm_no);

  BPSK bpsk;
  Convolutional_Code code;
  ivec generator = cfg.get_value("gen","0133,0165,0171");
  int L = cfg.get_value("L", 7);
  code.set_generator_polynomials(generator, L);
  
  int no_bits = cfg.get_value("no_bits", 100);
  bvec bits, encoded_bits, decoded_bits;
  vec tx_signal, rx_signal;
  
  int i = 0;
  do{
    berc.clear();
    double snr = inv_dB(cfg.get_value("snr", 7));
    double N0 = pow(snr,-1.0);
    bits=randb(no_bits);
    code.encode_tail(bits, encoded_bits);
    tx_signal = bpsk.modulate_bits(encoded_bits);
    rx_signal = tx_signal + sqrt(N0)*randn(tx_signal.size());
    code.decode_tail(rx_signal, decoded_bits);
    berc.count(bits, decoded_bits);
    errorrate(i) = berc.get_errorrate();
  } while ( mod(cfg.counter(++i),swp_prm_no) );
  
  tt.toc();
   time_t tm;
  time(&tm);
  string strtime(ctime(&tm));

  it_file ff;                    //For saving the results to file
  string rslt_file_name( id+".dat" );   // !out

  cout << "SNR = " << swp_prm << endl;
  cout << "BER = " << errorrate << endl;
  cout << "Saving results to " << rslt_file_name << endl;
  cout << endl;

  ff.open(rslt_file_name);
  ff << Name("time") << (string &)strtime;
  ff << Name("swp_prm") << swp_prm;
  ff << Name("ber") << errorrate;
  ff.close();

  return 0;
}
